Class: FileMonitoring::FileStatEnum
- Inherits:
-
Object
- Object
- FileMonitoring::FileStatEnum
- Defined in:
- lib/file_monitoring/monitor_path.rb
Overview
Enum-like structure that includes possible filesystem entities (files/directories) states:
- NON_EXISTING - Entity that was treated during previous run, but absent currently
- NEW - Entity that was found and added to control during this run
- CHANGED - State was changed between two checks
- UNCHANGED - Opposite to CHANGED
- STABLE - Entity is in the UNCHANGED state for a defined (by user) number of iterations
Constant Summary collapse
- NON_EXISTING =
"NON_EXISTING"- NEW =
"NEW"- CHANGED =
"CHANGED"- UNCHANGED =
"UNCHANGED"- STABLE =
"STABLE"